﻿// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF
// ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO
// THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
// PARTICULAR PURPOSE.
//
// Copyright (c) Microsoft Corporation. All rights reserved

using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using Windows.ApplicationModel.Background;
using Windows.Foundation;
using Windows.Graphics.Display;
using Windows.Storage;
using Windows.System;
using Windows.UI.Core;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Media;

namespace BackgroundTaskSample
{
    partial class MainPage
    {

        private const string SampleBackgroundTaskEntryPoint = "BackgroundTask.SampleBackgroundTask";
        private const string SampleBackgroundTaskName = "SampleBackgroundTask";
        private const string SampleBackgroundTaskWithConditionName = "SampleBackgroundTaskWithCondition";
        private CoreDispatcher SampleDispatcher;
        private const string ServicingCompleteTaskEntryPoint = "BackgroundTask.ServicingComplete";
        private const string ServicingCompleteTaskName = "ServicingCompleteTask";
        private const string TimeTriggeredTaskName = "TimeTriggeredTask";

        public MainPage()
        {
            InitializeComponent();
            ScenarioList.SelectionChanged += new SelectionChangedEventHandler(ScenarioList_SelectionChanged);

            //
            // Take a reference to the main window dispatcher object to the UI.
            //
            SampleDispatcher = Window.Current.CoreWindow.Dispatcher;

            // Starting scenario is the first or based upon a previous selection
            ListBoxItem startingScenario = null;
            if (SuspensionManager.SessionState.ContainsKey("SelectedScenario"))
            {
                String selectedScenarioName = SuspensionManager.SessionState["SelectedScenario"] as string;
                startingScenario = this.FindName(selectedScenarioName) as ListBoxItem;
            }

            ScenarioList.SelectedItem = startingScenario != null ? startingScenario : Scenario1;
            
            Scenario1Init();
            Scenario2Init();
            Scenario3Init();
            Scenario4Init();

            Windows.UI.ViewManagement.ApplicationView.GetForCurrentView().ViewStateChanged += new TypedEventHandler<Windows.UI.ViewManagement.ApplicationView, Windows.UI.ViewManagement.ApplicationViewStateChangedEventArgs>(MainPage_ViewStateChanged);

            try
            {
                //
                // Initialize state-based registration of currently registered background tasks.
                //
                InitializeRegisteredBackgroundTasks();

                //
                // Register a background task for the servicing-complete system event.
                // This event is triggered when the application is updated.
                //
                RegisterServicingCompleteTask();
            }
            catch (Exception ex)
            {
                Error.Text = ex.ToString();
            }
        }

        //
        // Register a background task with the specified taskEntryPoint, name, trigger,
        // and condition (optional).
        //
        private void RegisterBackgroundTask(String taskEntryPoint, String name, IBackgroundTrigger trigger, IBackgroundCondition condition)
        {
            // BackgroundTask.SampleBackgroundTask
            var builder = new BackgroundTaskBuilder();

            builder.Name = name;
            builder.TaskEntryPoint = taskEntryPoint;
            builder.SetTrigger(trigger);

            if (condition != null)
            {
                builder.AddCondition(condition);
            }

            BackgroundTaskRegistration task = builder.Register();

            AttachProgressAndCompletedHandlers(task);

            UpdateBackgroundTaskUI(name, true);
        }

        //
        // Associate progress and completed event handlers with the specified background task.
        //
        private void AttachProgressAndCompletedHandlers(IBackgroundTaskRegistration task)
        {
            task.Progress += new BackgroundTaskProgressEventHandler(OnProgress);
            task.Completed += new BackgroundTaskCompletedEventHandler(OnCompleted);
        }

        //
        // Registers a background task for the servicing-complete system event.
        // This event occurs when the application is updated.
        //
        private void RegisterServicingCompleteTask()
        {
            //
            // Check whether the servicing-complete background task is already registered.
            //
            foreach (var cur in BackgroundTaskRegistration.AllTasks)
            {
                if (cur.Value.Name == ServicingCompleteTaskName)
                {
                    //
                    // The task is already registered.
                    //
                    UpdateBackgroundTaskUI(ServicingCompleteTaskName, true);
                    return;
                }
            }

            //
            // The servicing-complete background task is not already registered.
            //
            RegisterBackgroundTask(ServicingCompleteTaskEntryPoint,
                                   ServicingCompleteTaskName,
                                   new SystemTrigger(SystemTriggerType.ServicingComplete, false),
                                   null);
        }

        //
        // Registers a background task for a TimeZoneChange event.
        //
        private void RegisterSampleBackgroundTask(object sender, RoutedEventArgs args)
        {
            RegisterBackgroundTask(SampleBackgroundTaskEntryPoint,
                                   SampleBackgroundTaskName,
                                   new SystemTrigger(SystemTriggerType.TimeZoneChange, false),
                                   null);
        }

        //
        // Registers a background task with an UserPresent system condition for TimeZoneChange.
        // The system condition requires that the user is logged on and not idle before the background task can run.
        //
        private void RegisterSampleBackgroundTaskWithCondition(object sender, RoutedEventArgs args)
        {
            RegisterBackgroundTask(SampleBackgroundTaskEntryPoint,
                                   SampleBackgroundTaskWithConditionName,
                                   new SystemTrigger(SystemTriggerType.TimeZoneChange, false),
                                   new SystemCondition(SystemConditionType.UserPresent));
        }

        //
        // Requests loskcreen access and registers a background task with a periodic time trigger event
        // that will fire every fifteen minutes until cancelled.
        //
        public async void RegisterTimeTriggeredBackgroundTask(object sender, RoutedEventArgs args)
        {
            await RequestLockScreenAccessAsync();
        }

        async Task RequestLockScreenAccessAsync()
        {
            BackgroundAccessStatus status = await BackgroundExecutionManager.RequestAccessAsync();
            switch (status)
            {
                case BackgroundAccessStatus.AllowedWithRealTimeConnectivity:
                case BackgroundAccessStatus.AllowedWithoutRealTimeConnectivity:
                    RegisterBackgroundTask(SampleBackgroundTaskEntryPoint,
                                           TimeTriggeredTaskName,
                                           new TimeTrigger(15, false),
                                           null);
                    break;
                default:
                    break;
            }
        }

        //
        // Handle currently registered background tasks on application startup.
        //
        public void InitializeRegisteredBackgroundTasks()
        {
            //
            // Initialize UI elements based on currently registered background tasks
            // and associate background task progress and completed event
            // handlers with each background task.
            //
            UpdateSampleBackgroundTaskUIState(false);
            UpdateSampleBackgroundTaskWithConditionUIState(false);
            UpdateServicingCompleteTaskUIState(false);
            UpdateTimeTriggeredBackgroundTaskUIState(false);

            foreach (var cur in BackgroundTaskRegistration.AllTasks)
            {
                UpdateBackgroundTaskUI(cur.Value.Name, true);
                cur.Value.Progress += new BackgroundTaskProgressEventHandler(OnProgress);
                cur.Value.Completed += new BackgroundTaskCompletedEventHandler(OnCompleted);
            }
        }

        //
        // Handle background task completion.
        //
        private void OnCompleted(IBackgroundTaskRegistration sender, BackgroundTaskCompletedEventArgs e)
        {
            //
            // Update the UI with progress reported by the background task.
            //
            SampleDispatcher.InvokeAsync(CoreDispatcherPriority.Normal,
                                         (task, args) =>
                                         {
                                             try
                                             {
                                                 var taskCompletion = task as IBackgroundTaskRegistration;
                                                 var completionArgs = args.Context as BackgroundTaskCompletedEventArgs;

                                                 if ((taskCompletion != null) && (args != null))
                                                 {
                                                     //
                                                     // If the background task threw an exception, display the exception in
                                                     // the error text box.
                                                     //
                                                     if (completionArgs.Status != null)
                                                     {
                                                         throw completionArgs.Status;
                                                     }

                                                     //
                                                     // Update the UI with the completion status of the background task
                                                     // The Run method of the background task sets this status. 
                                                     //
                                                     var key = taskCompletion.TaskId.ToString();
                                                     var settings = ApplicationData.Current.LocalSettings;
                                                     switch (taskCompletion.Name)
                                                     {
                                                         case SampleBackgroundTaskName:
                                                             SampleBackgroundTaskStatus.Text = settings.Values[key].ToString();
                                                             break;
                                                         case SampleBackgroundTaskWithConditionName:
                                                             SampleBackgroundTaskWithConditionStatus.Text = settings.Values[key].ToString();
                                                             break;
                                                         case ServicingCompleteTaskName:
                                                             ServicingCompleteStatus.Text = settings.Values[key].ToString();
                                                             break;
                                                         case TimeTriggeredTaskName:
                                                             TimeTriggeredBackgroundTaskStatus.Text = settings.Values[key].ToString();
                                                             break;
                                                     }
                                                 }
                                             }
                                             catch (Exception ex)
                                             {
                                                 Error.Text = ex.ToString();
                                             }
                                         },
                                         sender,
                                         e);
        }

        //
        // Handle background task progress.
        //
        private void OnProgress(IBackgroundTaskRegistration sender, BackgroundTaskProgressEventArgs e)
        {
            //
            // Update the UI with progress reported by the background task.
            //
            SampleDispatcher.InvokeAsync(CoreDispatcherPriority.Normal,
                                         (task, args) =>
                                         {
                                             var taskRegistration = task as IBackgroundTaskRegistration;
                                             var progressArgs = args.Context as BackgroundTaskProgressEventArgs;

                                             if ((task != null) && (args != null))
                                             {
                                                 switch (taskRegistration.Name)
                                                 {
                                                     case SampleBackgroundTaskName:
                                                         SampleBackgroundTaskProgress.Text = "Progress: " + progressArgs.Progress + "%";
                                                         break;
                                                     case SampleBackgroundTaskWithConditionName:
                                                         SampleBackgroundTaskWithConditionProgress.Text = "Progress: " + progressArgs.Progress + "%";
                                                         break;
                                                     case ServicingCompleteTaskName:
                                                         ServicingCompleteProgress.Text = "Progress: " + progressArgs.Progress + "%";
                                                         break;
                                                     case TimeTriggeredTaskName:
                                                         TimeTriggeredBackgroundTaskProgress.Text = "Progress: " + progressArgs.Progress + "%";
                                                         break;
                                                 }
                                             }
                                         },
                                         sender,
                                         e);
        }

        //
        // Unregister background tasks with specified name.
        //
        private void UnregisterBackgroundTasks(string name)
        {
            //
            // Loop through all background tasks and unregister any with SampleBackgroundTaskName or
            // SampleBackgroundTaskWithConditionName.
            //
            foreach (var cur in BackgroundTaskRegistration.AllTasks)
            {
                if (cur.Value.Name == name)
                {
                    cur.Value.Unregister(true);
                }
            }

            UpdateBackgroundTaskUI(name, false);
        }

        private void UnregisterSampleBackgroundTaskWithCondition(object sender, RoutedEventArgs args)
        {
            UnregisterBackgroundTasks(SampleBackgroundTaskWithConditionName);
        }

        private void UnregisterSampleBackgroundTask(object sender, RoutedEventArgs args)
        {
            UnregisterBackgroundTasks(SampleBackgroundTaskName);
        }

        public void UnregisterTimeTriggeredBackgroundTask(object sender, RoutedEventArgs args)
        {
            UnregisterBackgroundTasks(TimeTriggeredTaskName);
        }

        void MainPage_ViewStateChanged(Windows.UI.ViewManagement.ApplicationView sender, Windows.UI.ViewManagement.ApplicationViewStateChangedEventArgs args)
        {
            switch (args.ViewState)
            {
                case Windows.UI.ViewManagement.ApplicationViewState.Filled:
                    VisualStateManager.GoToState(this, "Fill", false);
                    break;
                case Windows.UI.ViewManagement.ApplicationViewState.FullScreenLandscape:
                    VisualStateManager.GoToState(this, "Full", false);
                    break;
                case Windows.UI.ViewManagement.ApplicationViewState.Snapped:
                    VisualStateManager.GoToState(this, "Snapped", false);
                    break;
                case Windows.UI.ViewManagement.ApplicationViewState.FullScreenPortrait:
                    VisualStateManager.GoToState(this, "Portrait", false);
                    break;
                default:
                    break;
            }
        }

        #region Scenario Specific Code

        void Scenario1Init()
        {
        }

        void Scenario1Reset()
        {
        }

        void Scenario2Init()
        {
        }

        void Scenario2Reset()
        {
        }

        void Scenario3Init()
        {
        }

        void Scenario3Reset()
        {
        }

        void Scenario4Init()
        {
        }
        void Scenario4Reset()
        {
        }

        #endregion

        void ScenarioList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            ResetAll();
            if (ScenarioList.SelectedItem == Scenario1)
            {
                Scenario1Input.Visibility = Visibility.Visible;
                Scenario1Output.Visibility = Visibility.Visible;
            }
            else if (ScenarioList.SelectedItem == Scenario2)
            {
                Scenario2Input.Visibility = Visibility.Visible;
                Scenario2Output.Visibility = Visibility.Visible;
            }
            else if (ScenarioList.SelectedItem == Scenario3)
            {
                Scenario3Input.Visibility = Visibility.Visible;
                Scenario3Output.Visibility = Visibility.Visible;
            }
            else if (ScenarioList.SelectedItem == Scenario4)
            {
                Scenario4Input.Visibility = Visibility.Visible;
                Scenario4Output.Visibility = Visibility.Visible;
            }

            if (ScenarioList.SelectedItem != null)
            {
                ListBoxItem selectedListBoxItem = ScenarioList.SelectedItem as ListBoxItem;
                SuspensionManager.SessionState["SelectedScenario"] = selectedListBoxItem.Name;
            }
        }

        public void ResetAll()
        {
            Scenario1Input.Visibility = Visibility.Collapsed;
            Scenario1Output.Visibility = Visibility.Collapsed;
            Scenario1Reset();
            Scenario2Input.Visibility = Visibility.Collapsed;
            Scenario2Output.Visibility = Visibility.Collapsed;
            Scenario2Reset();
            Scenario3Input.Visibility = Visibility.Collapsed;
            Scenario3Output.Visibility = Visibility.Collapsed;
            Scenario3Reset();
            Scenario4Input.Visibility = Visibility.Collapsed;
            Scenario4Output.Visibility = Visibility.Collapsed;
            Scenario4Reset();
        }

        async void Footer_Click(object sender, RoutedEventArgs e)
        {
            await Windows.System.Launcher.LaunchUriAsync(new Uri(((HyperlinkButton)sender).Tag.ToString()));
        }

        private void UpdateBackgroundTaskUI(String name, bool registered)
        {
            if (name == SampleBackgroundTaskName)
            {
                UpdateSampleBackgroundTaskUIState(registered);
            }
            else if (name == SampleBackgroundTaskWithConditionName)
            {
                UpdateSampleBackgroundTaskWithConditionUIState(registered);
            }
            else if (name == ServicingCompleteTaskName)
            {
                UpdateServicingCompleteTaskUIState(registered);
            }
            else if (name == TimeTriggeredTaskName)
            {
                UpdateTimeTriggeredBackgroundTaskUIState(registered);
            }
        }

        //
        // Updates button text in the sample application UI.
        //
        private void UpdateSampleBackgroundTaskUIState(bool Registered)
        {
            if (Registered)
            {
                SampleBackgroundTaskStatus.Text = "Registered";
                RegisterSampleBackgroundTaskButton.IsEnabled = false;
                UnregisterSampleBackgroundTaskButton.IsEnabled = true;
            }
            else
            {
                SampleBackgroundTaskStatus.Text = "Unregistered";
                RegisterSampleBackgroundTaskButton.IsEnabled = true;
                UnregisterSampleBackgroundTaskButton.IsEnabled = false;
            }
        }

        //
        // Updates button text in the sample application UI.
        //
        private void UpdateSampleBackgroundTaskWithConditionUIState(bool Registered)
        {
            if (Registered)
            {
                SampleBackgroundTaskWithConditionStatus.Text = "Registered";
                RegisterSampleBackgroundTaskWithConditionButton.IsEnabled = false;
                UnregisterSampleBackgroundTaskWithConditionButton.IsEnabled = true;
            }
            else
            {
                SampleBackgroundTaskWithConditionStatus.Text = "Unregistered";
                RegisterSampleBackgroundTaskWithConditionButton.IsEnabled = true;
                UnregisterSampleBackgroundTaskWithConditionButton.IsEnabled = false;
            }
        }

        private void UpdateServicingCompleteTaskUIState(bool Registered)
        {
            if (Registered)
            {
                ServicingCompleteStatus.Text = "Registered";
            }
            else
            {
                ServicingCompleteStatus.Text = "Unregistered";
            }
        }

        private void UpdateTimeTriggeredBackgroundTaskUIState(bool Registered)
        {
            if (Registered)
            {
                TimeTriggeredBackgroundTaskStatus.Text = "Registered";
                RegisterTimeTriggeredBackgroundTaskButton.IsEnabled = false;
                UnregisterTimeTriggeredBackgroundTaskButton.IsEnabled = true;
            }
            else
            {
                TimeTriggeredBackgroundTaskStatus.Text = "Unregistered";
                RegisterTimeTriggeredBackgroundTaskButton.IsEnabled = true;
                UnregisterTimeTriggeredBackgroundTaskButton.IsEnabled = false;
            }
        }
    }
}
